import unittest

import requests_mock
from smsframework import Gateway, OutgoingMessage

from smsframework_africastalking import AfricasTalkingProvider
from smsframework_africastalking.error import AfricasTalkingProviderError, InvalidNumberError


def _mock_response(status_code, msg_response):
    """
    Decorator function to handle mocking responses to API calls

    :param status_code: (int) Mocked response status code
    :param msg_response: (dict) Mocked message response details
    """
    def decorator(test_func):
        def wrapper(*args, **kwargs):
            response = {
                'SMSMessageData': {
                    'Recipients': [msg_response]
                }
            }
            with requests_mock.mock() as mocker:
                mocker.post(
                    'https://api.sandbox.africastalking.com/version1/messaging',
                    headers={'content-type': 'application/json'},
                    status_code=status_code,
                    json=response
                )
                test_func(*args, **kwargs)
        return wrapper
    return decorator


class AfricasTalkingProviderTest(unittest.TestCase):
    def setUp(self):
        """Initialize AfricasTalkingProvider"""
        self.gw = gw = Gateway()
        gw.add_provider(
            'africas_talking',
            AfricasTalkingProvider,
            username='sandbox',
            api_key='api_key'
        )

    @_mock_response(200, {'status': 'Success', 'messageId': '001'})
    def test_send_success(self):
        """Test a successful AfricasTalking SMS send"""
        message_out = OutgoingMessage(
            '+254789789789',
            'Hello Kenya',
            provider='africas_talking'
        ).params(
            target_country='KE'
        )
        message_back = self.gw.send(message_out)
        self.assertEqual(message_back.msgid, '001')

    @_mock_response(500, {'status': 'Failed'})
    def test_send_failure(self):
        """Test a failing AfricasTalking SMS send"""
        message_out = OutgoingMessage(
            '+254789789789',
            'Hello Kenya',
            provider='africas_talking'
        ).params(
            target_country='KE'
        )
        self.assertRaises(
            AfricasTalkingProviderError,
            self.gw.send,
            message_out
        )

    @_mock_response(403, {'status': 'InvalidPhoneNumber'})
    def test_bad_number_failure(self):
        """Test a failing AfricasTalking SMS send (bad phone number)"""
        message_out = OutgoingMessage(
            '+254789789789',
            'Hello Rwanda',
            provider='africas_talking'
        ).params(
            target_country='RW'
        )
        self.assertRaises(
            InvalidNumberError,
            self.gw.send,
            message_out
        )



import logging
import json

import africastalking
import phonenumbers
from smsframework import IProvider

from .error import AfricasTalkingProviderError, InvalidNumberError


class AfricasTalkingProvider(IProvider):
    """Africa's Talking Provider"""

    def __init__(self, gateway, name, username, api_key):
        """
        Configure AfricasTalking Provider

        :param gateway: (smsframework.Gateway) Passed automatically by gateway
         on add_provider call
        :param name: (str) Uniquely identify the instance of
         AfricasTalkingProvider registered to the gateway
        :param username: (str) Username for your AfricasTalking account
        :param api_key: (str) API key for your AfricasTalking account
        """
        super().__init__(gateway, name)

        africastalking.initialize(username, api_key)
        self.sms_client = africastalking.SMS

    def send(self, message):
        """
        Send a single text message.
        TODO: Add a batch send feature.

        :param message: (smsframework.OutgoingMessage) The message to send.
        :return: (smsframework.OutgoingMessage) The sent message, updated with
         msgid.
        """

        logging.debug('smsframework africastalking sending sms')

        target_country = message.provider_params['target_country']

        try:
            phone_number = phonenumbers.parse(message.dst, target_country)
        except:
            raise InvalidNumberError(message.dst, 'Unable to Parse Number')

        number_is_valid = phonenumbers.is_valid_number_for_region(
            phone_number, target_country
        )
        if number_is_valid is False:
            raise InvalidNumberError(
                message.dst,
                'Invalid Phone Number for Target Country',
                target_country=target_country
            )

        try:
            formatted_number = phonenumbers.format_number(
                phone_number,
                phonenumbers.PhoneNumberFormat.E164
            )
        except:
            InvalidNumberError(
                message.dst,
                'Unable to Parse Phone Number'
            )

        try:
            api_response = self.sms_client.send(
                message.body,
                [formatted_number],
                message.provider_options.senderId
            )
        except Exception as e:
            error = json.loads(e.args[0])
            error_status = error['SMSMessageData']['Recipients'][0]['status']

            logging.error('smsframework africastalking sms failed')

            if error_status == 'InvalidPhoneNumber':
                raise InvalidNumberError(formatted_number)
            else:
                raise AfricasTalkingProviderError(error_status)

        sent_message = api_response['SMSMessageData']['Recipients'][0]
        message.msgid = sent_message['messageId']

        return message



from smsframework.exc import ProviderError, RequestError


class AfricasTalkingProviderError(ProviderError):

    def __init__(self, message):
        super().__init__(message)


class InvalidNumberError(AfricasTalkingProviderError):

    def __init__(self, number, message='Invalid Phone Number', **kwargs):
        super().__init__(message)

        self.number = number

        for key, value in kwargs.items():
            setattr(self, key, value)



#!/usr/bin/env python

import setuptools

with open('README.md', 'r') as fh:

    long_description = fh.read()

setuptools.setup(
    name='smsframework-africastalking',
    version='1.0.3',
    author='BBOXX',
    author_email='j.lynch@bboxx.co.uk',
    description='SMS framework: Africa\'s Talking provider',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/BBOXX/py-smsframework-africastalking',
    packages=setuptools.find_packages(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3'
    ],
    keywords='sms message send africa africastalking smsframework provider'
)
